@@ -27,6 +27,7 @@
  *    POSSIBILITY OF SUCH DAMAGE.
  *
  */
+
 namespace OPNsense\Base\FieldTypes;
 
 use OPNsense\Base\Validators\NetworkValidator;
@@ -51,6 +52,11 @@ class NetworkField extends BaseField
      */
     protected $internalNetMaskRequired = false;
 
+    /**
+     * @var bool marks if net mask is (dis)allowed
+     */
+    protected $internalNetMaskAllowed = true;
+
     /**
      * @var null when multiple values could be provided at once, specify the split character
      */
@@ -93,6 +99,15 @@ class NetworkField extends BaseField
         }
     }
 
+    /**
+     * setter for net mask required
+     * @param integer $value
+     */
+    public function setNetMaskAllowed($value)
+    {
+        $this->internalNetMaskAllowed = (trim(strtoupper($value)) == "Y");
+    }
+
     /**
      * setter for address family
      * @param $value address family [ipv4, ipv6, empty for all]
@@ -170,6 +185,7 @@ class NetworkField extends BaseField
                     'message' => $this->internalValidationMessage,
                     'split' => $this->internalFieldSeparator,
                     'netMaskRequired' => $this->internalNetMaskRequired,
+                    'netMaskAllowed' => $this->internalNetMaskAllowed,
                     'version' => $this->internalAddressFamily
                     ));
             }
