@@ -1,14 +1,12 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
 Copyright (c) 2014-2019 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
+from __future__ import print_function
 
-import BaseHTTPServer
-import cStringIO
 import datetime
-import httplib
 import glob
 import gzip
 import hashlib
@@ -18,13 +16,10 @@ import mimetypes
 import os
 import re
 import socket
-import SocketServer
 import subprocess
 import threading
 import time
 import traceback
-import urllib
-import urlparse
 
 from core.addr import addr_to_int
 from core.addr import int_to_addr
@@ -33,6 +28,7 @@ from core.attribdict import AttribDict
 from core.common import get_regex
 from core.common import ipcat_lookup
 from core.common import worst_asns
+from core.compat import xrange
 from core.enums import HTTP_HEADER
 from core.settings import config
 from core.settings import CONTENT_EXTENSIONS_EXCLUSIONS
@@ -41,6 +37,7 @@ from core.settings import DISABLED_CONTENT_EXTENSIONS
 from core.settings import DISPOSED_NONCES
 from core.settings import HTML_DIR
 from core.settings import HTTP_TIME_FORMAT
+from core.settings import IS_WIN
 from core.settings import MAX_NOFILE
 from core.settings import NAME
 from core.settings import PING_RESPONSE
@@ -50,9 +47,14 @@ from core.settings import SESSION_COOKIE_FLAG_SAMESITE
 from core.settings import SESSION_EXPIRATION_HOURS
 from core.settings import SESSION_ID_LENGTH
 from core.settings import SESSIONS
-from core.settings import TRAILS_FILE
 from core.settings import UNAUTHORIZED_SLEEP_TIME
+from core.settings import UNICODE_ENCODING
 from core.settings import VERSION
+from thirdparty import six
+from thirdparty.six.moves import BaseHTTPServer as _BaseHTTPServer
+from thirdparty.six.moves import http_client as _http_client
+from thirdparty.six.moves import socketserver as _socketserver
+from thirdparty.six.moves import urllib as _urllib
 
 try:
     # Reference: https://bugs.python.org/issue7980
@@ -72,14 +74,14 @@ def start_httpd(address=None, port=None, join=False, pem=None):
     Starts HTTP server
     """
 
-    class ThreadingServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
+    class ThreadingServer(_socketserver.ThreadingMixIn, _BaseHTTPServer.HTTPServer):
         def server_bind(self):
             self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-            BaseHTTPServer.HTTPServer.server_bind(self)
+            _BaseHTTPServer.HTTPServer.server_bind(self)
 
         def finish_request(self, *args, **kwargs):
             try:
-                BaseHTTPServer.HTTPServer.finish_request(self, *args, **kwargs)
+                _BaseHTTPServer.HTTPServer.finish_request(self, *args, **kwargs)
             except:
                 if config.SHOW_DEBUG:
                     traceback.print_exc()
@@ -89,7 +91,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             import OpenSSL  # python-openssl
 
             ThreadingServer.__init__(self, server_address, HandlerClass)
-            ctx = OpenSSL.SSL.Context(OpenSSL.SSL.TLSv1_METHOD)
+            ctx = OpenSSL.SSL.Context(OpenSSL.SSL.TLSv1_2_METHOD)
             ctx.use_privatekey_file(pem)
             ctx.use_certificate_file(pem)
             self.socket = OpenSSL.SSL.Connection(ctx, socket.socket(self.address_family, self.socket_type))
@@ -103,7 +105,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                 if config.SHOW_DEBUG:
                     traceback.print_exc()
 
-    class ReqHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+    class ReqHandler(_BaseHTTPServer.BaseHTTPRequestHandler):
         def do_GET(self):
             path, query = self.path.split('?', 1) if '?' in self.path else (self.path, "")
             params = {}
@@ -111,10 +113,10 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             skip = False
 
             if hasattr(self, "data"):
-                params.update(urlparse.parse_qs(self.data))
+                params.update(_urllib.parse.parse_qs(self.data))
 
             if query:
-                params.update(urlparse.parse_qs(query))
+                params.update(_urllib.parse.parse_qs(query))
 
             for key in params:
                 if params[key]:
@@ -139,7 +141,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                 if any((config.IP_ALIASES,)) and self.path.split('?')[0] == "/js/main.js":
                     content = open(path, "rb").read()
                     content = re.sub(r"\bvar IP_ALIASES =.+", "var IP_ALIASES = {%s};" % ", ".join('"%s": "%s"' % (_.split(':', 1)[0].strip(), _.split(':', 1)[-1].strip()) for _ in config.IP_ALIASES), content)
-                    self.send_response(httplib.OK)
+                    self.send_response(_http_client.OK)
                 elif ".." not in os.path.relpath(path, HTML_DIR) and os.path.isfile(path) and (extension not in DISABLED_CONTENT_EXTENSIONS or os.path.split(path)[-1] in CONTENT_EXTENSIONS_EXCLUSIONS):
                     mtime = time.gmtime(os.path.getmtime(path))
                     if_modified_since = self.headers.get(HTTP_HEADER.IF_MODIFIED_SINCE)
@@ -147,20 +149,22 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     if if_modified_since and extension not in (".htm", ".html"):
                         if_modified_since = [_ for _ in if_modified_since.split(';') if _.upper().endswith("GMT")][0]
                         if time.mktime(mtime) <= time.mktime(time.strptime(if_modified_since, HTTP_TIME_FORMAT)):
-                            self.send_response(httplib.NOT_MODIFIED)
+                            self.send_response(_http_client.NOT_MODIFIED)
                             self.send_header(HTTP_HEADER.CONNECTION, "close")
                             skip = True
 
                     if not skip:
                         content = open(path, "rb").read()
                         last_modified = time.strftime(HTTP_TIME_FORMAT, mtime)
-                        self.send_response(httplib.OK)
+                        self.send_response(_http_client.OK)
                         self.send_header(HTTP_HEADER.CONNECTION, "close")
                         self.send_header(HTTP_HEADER.CONTENT_TYPE, mimetypes.guess_type(path)[0] or "application/octet-stream")
                         self.send_header(HTTP_HEADER.LAST_MODIFIED, last_modified)
-                        self.send_header(HTTP_HEADER.CONTENT_SECURITY_POLICY, "default-src 'none'; style-src 'self' 'unsafe-inline'; img-src 'self'; " +
-                                                                              "script-src 'self' 'unsafe-eval' https://stat.ripe.net; connect-src 'self'; form-action 'self'; " +
-                                                                              "font-src 'self'; frame-src *; worker-src 'self'; block-all-mixed-content;")
+
+                        # For CSP policy directives see: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/
+                        self.send_header(HTTP_HEADER.CONTENT_SECURITY_POLICY, "default-src 'self'; style-src 'self' 'unsafe-inline'; img-src *; " +
+                                                                              "script-src 'self' 'unsafe-eval' https://stat.ripe.net; " +
+                                                                              "frame-src *; object-src 'none'; block-all-mixed-content;")
                         if extension not in (".htm", ".html"):
                             self.send_header(HTTP_HEADER.EXPIRES, "Sun, 17-Jan-2038 19:14:07 GMT")        # Reference: http://blog.httpwatch.com/2007/12/10/two-simple-rules-for-http-caching/
                             self.send_header(HTTP_HEADER.CACHE_CONTROL, "max-age=3600, must-revalidate")  # Reference: http://stackoverflow.com/a/5084555
@@ -168,20 +172,23 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                             self.send_header(HTTP_HEADER.CACHE_CONTROL, "no-cache")
 
                 else:
-                    self.send_response(httplib.NOT_FOUND)
+                    self.send_response(_http_client.NOT_FOUND)
                     self.send_header(HTTP_HEADER.CONNECTION, "close")
-                    content = '<!DOCTYPE html><html lang="en"><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested URL %s was not found on this server.</p></body></html>' % self.path.split('?')[0]
+                    content = b'<!DOCTYPE html><html lang="en"><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested URL %s was not found on this server.</p></body></html>' % self.path.split('?')[0]
 
             if content is not None:
-                for match in re.finditer(r"<\!(\w+)\!>", content):
+                if isinstance(content, six.text_type):
+                    content = content.encode(UNICODE_ENCODING)
+
+                for match in re.finditer(b"<\\!(\\w+)\\!>", content):
                     name = match.group(1)
                     _ = getattr(self, "_%s" % name.lower(), None)
                     if _:
                         content = self._format(content, **{ name: _() })
 
-                if "gzip" in self.headers.getheader(HTTP_HEADER.ACCEPT_ENCODING, ""):
+                if "gzip" in self.headers.get(HTTP_HEADER.ACCEPT_ENCODING):
                     self.send_header(HTTP_HEADER.CONTENT_ENCODING, "gzip")
-                    _ = cStringIO.StringIO()
+                    _ = six.BytesIO()
                     compress = gzip.GzipFile("", "w+b", 9, _)
                     compress._stream = _
                     compress.write(content)
@@ -197,12 +204,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                 self.wfile.write(content)
 
             self.wfile.flush()
-            self.wfile.close()
 
         def do_POST(self):
-            length = self.headers.getheader(HTTP_HEADER.CONTENT_LENGTH)
-            data = self.rfile.read(int(length))
-            data = urllib.unquote_plus(data)
+            length = self.headers.get(HTTP_HEADER.CONTENT_LENGTH)
+            data = self.rfile.read(int(length)).decode(UNICODE_ENCODING)
+            data = _urllib.parse.unquote_plus(data)
             self.data = data
             self.do_GET()
 
@@ -222,6 +228,9 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         else:
                             del SESSIONS[session]
 
+            if retval is None and not config.USERS:
+                retval = AttribDict({"username": "?"})
+
             return retval
 
         def delete_session(self):
@@ -239,7 +248,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
         def end_headers(self):
             if not hasattr(self, "_headers_ended"):
-                BaseHTTPServer.BaseHTTPRequestHandler.end_headers(self)
+                _BaseHTTPServer.BaseHTTPRequestHandler.end_headers(self)
                 self._headers_ended = True
 
         def log_message(self, format, *args):
@@ -247,7 +256,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
         def finish(self):
             try:
-                BaseHTTPServer.BaseHTTPRequestHandler.finish(self)
+                _BaseHTTPServer.BaseHTTPRequestHandler.finish(self)
             except:
                 if config.SHOW_DEBUG:
                     traceback.print_exc()
@@ -255,6 +264,14 @@ def start_httpd(address=None, port=None, join=False, pem=None):
         def _version(self):
             return VERSION
 
+        def _logo(self):
+            if config.HEADER_LOGO:
+                retval = config.HEADER_LOGO
+            else:
+                retval = '<img src="images/mlogo.png" style="width: 25px">altrail'
+
+            return retval
+
         def _format(self, content, **params):
             if content:
                 for key, value in params.items():
@@ -273,7 +290,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         username, stored_hash, uid, netfilter = entry.split(':')
                         if username == params.get("username"):
                             try:
-                                if params.get("hash") == hashlib.sha256(stored_hash.strip() + params.get("nonce")).hexdigest():
+                                if params.get("hash") == hashlib.sha256((stored_hash.strip() + params.get("nonce")).encode(UNICODE_ENCODING)).hexdigest():
                                     valid = True
                                     break
                             except:
@@ -281,10 +298,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                                     traceback.print_exc()
 
             if valid:
-                session_id = os.urandom(SESSION_ID_LENGTH).encode("hex")
+                _ = os.urandom(SESSION_ID_LENGTH)
+                session_id = _.hex() if hasattr(_, "hex") else _.encode("hex")
                 expiration = time.time() + 3600 * SESSION_EXPIRATION_HOURS
 
-                self.send_response(httplib.OK)
+                self.send_response(_http_client.OK)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
 
                 cookie = "%s=%s; expires=%s; path=/; HttpOnly" % (SESSION_COOKIE_NAME, session_id, time.strftime(HTTP_TIME_FORMAT, time.gmtime(expiration)))
@@ -329,13 +347,13 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                 SESSIONS[session_id] = AttribDict({"username": username, "uid": uid, "netfilters": netfilters, "expiration": expiration, "client_ip": self.client_address[0]})
             else:
                 time.sleep(UNAUTHORIZED_SLEEP_TIME)
-                self.send_response(httplib.UNAUTHORIZED)
+                self.send_response(_http_client.UNAUTHORIZED)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
 
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
             content = "Login %s" % ("success" if valid else "failed")
 
-            if not subprocess.mswindows:
+            if not IS_WIN:
                 try:
                     subprocess.check_output("logger -p auth.info -t \"%s[%d]\" \"%s password for %s from %s port %s\"" % (NAME.lower(), os.getpid(), "Accepted" if valid else "Failed", params.get("username"), self.client_address[0], self.client_address[1]), stderr=subprocess.STDOUT, shell=True)
                 except Exception:
@@ -346,7 +364,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
         def _logout(self, params):
             self.delete_session()
-            self.send_response(httplib.FOUND)
+            self.send_response(_http_client.FOUND)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.LOCATION, "/")
 
@@ -354,7 +372,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             session = self.get_session()
             username = session.username if session else ""
 
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
@@ -364,11 +382,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             session = self.get_session()
 
             if session is None:
-                self.send_response(httplib.UNAUTHORIZED)
+                self.send_response(_http_client.UNAUTHORIZED)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
                 return None
 
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
@@ -385,14 +403,14 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     traceback.print_exc()
 
         def _trails(self, params):
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
-            return open(TRAILS_FILE, "rb").read()
+            return open(config.TRAILS_FILE, "rb").read()
 
         def _ping(self, params):
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
@@ -402,7 +420,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             session = self.get_session()
 
             if session is None:
-                self.send_response(httplib.UNAUTHORIZED)
+                self.send_response(_http_client.UNAUTHORIZED)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
                 return None
 
@@ -421,7 +439,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         range_handle = open(event_log_path, "rb")
                         log_exists = True
                 except ValueError:
-                    print "[!] invalid date format in request"
+                    print("[!] invalid date format in request")
                     log_exists = False
             else:
                 logs_data = ""
@@ -440,7 +458,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     range_handle = io.BytesIO(logs_data)
                     log_exists = True
                 except ValueError:
-                    print "[!] invalid date format in request"
+                    print("[!] invalid date format in request")
                     log_exists = False
 
             if log_exists:
@@ -461,17 +479,17 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
                         if session.netfilters is None:
                             session.range_handle.seek(start)
-                            self.send_response(httplib.PARTIAL_CONTENT)
+                            self.send_response(_http_client.PARTIAL_CONTENT)
                             self.send_header(HTTP_HEADER.CONNECTION, "close")
                             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
                             self.send_header(HTTP_HEADER.CONTENT_RANGE, "bytes %d-%d/%d" % (start, end, total))
                             content = session.range_handle.read(size)
                         else:
-                            self.send_response(httplib.OK)
+                            self.send_response(_http_client.OK)
                             self.send_header(HTTP_HEADER.CONNECTION, "close")
                             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
-                            buffer, addresses, netmasks, regex = cStringIO.StringIO(), set(), [], ""
+                            buffer, addresses, netmasks, regex = io.StringIO(), set(), [], ""
                             for netfilter in session.netfilters:
                                 if not netfilter:
                                     continue
@@ -479,10 +497,10 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                                     netmasks.append(netfilter)
                                 elif re.search(r"\A[\d.]+\Z", netfilter):
                                     addresses.add(netfilter)
-                                elif '\.' in netfilter:
+                                elif "\\." in netfilter:
                                     regex = r"\b(%s)\b" % netfilter
                                 else:
-                                    print "[!] invalid network filter '%s'" % netfilter
+                                    print("[!] invalid network filter '%s'" % netfilter)
                                     return
 
                             for line in session.range_handle:
@@ -529,7 +547,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                             session.range_handle = None
 
                 if size == -1:
-                    self.send_response(httplib.OK)
+                    self.send_response(_http_client.OK)
                     self.send_header(HTTP_HEADER.CONNECTION, "close")
                     self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
                     self.end_headers()
@@ -543,7 +561,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                                 self.wfile.write(data)
 
             else:
-                self.send_response(httplib.OK)  # instead of httplib.NO_CONTENT (compatibility reasons)
+                self.send_response(_http_client.OK)  # instead of _http_client.NO_CONTENT (compatibility reasons)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
                 if self.headers.get(HTTP_HEADER.RANGE):
                     self.send_header(HTTP_HEADER.CONTENT_RANGE, "bytes 0-0/0")
@@ -556,11 +574,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             session = self.get_session()
 
             if session is None:
-                self.send_response(httplib.UNAUTHORIZED)
+                self.send_response(_http_client.UNAUTHORIZED)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
                 return None
 
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "application/json")
 
@@ -595,10 +613,10 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         with open(filepath, "rb") as f:
                             content = f.read(io.DEFAULT_BUFFER_SIZE)
                             if size >= io.DEFAULT_BUFFER_SIZE:
-                                total = 1.0 * content.count('\n') * size / io.DEFAULT_BUFFER_SIZE
+                                total = 1.0 * content.count(b'\n') * size / io.DEFAULT_BUFFER_SIZE
                                 counts[timestamp] = int(round(total / 100) * 100)
                             else:
-                                counts[timestamp] = content.count('\n')
+                                counts[timestamp] = content.count(b'\n')
 
             return json.dumps(counts)
 
@@ -612,7 +630,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
     if ':' in (address or ""):
         address = address.strip("[]")
 
-        BaseHTTPServer.HTTPServer.address_family = socket.AF_INET6
+        _BaseHTTPServer.HTTPServer.address_family = socket.AF_INET6
 
         # Reference: https://github.com/squeaky-pl/zenchmarks/blob/master/vendor/twisted/internet/tcp.py
         _AI_NUMERICSERV = getattr(socket, "AI_NUMERICSERV", 0)
@@ -637,9 +655,9 @@ def start_httpd(address=None, port=None, join=False, pem=None):
         else:
             raise
 
-    print "[i] starting HTTP%s server at 'http%s://%s:%d/'" % ('S' if pem else "", 's' if pem else "", server.server_address[0], server.server_address[1])
+    print("[i] starting HTTP%s server at 'http%s://%s:%d/'" % ('S' if pem else "", 's' if pem else "", server.server_address[0], server.server_address[1]))
 
-    print "[o] running..."
+    print("[o] running...")
 
     if join:
         server.serve_forever()
