@@ -1,33 +1,33 @@
 <?php
 
 /*
-    Copyright (C) 2014-2015 Deciso B.V.
-    Copyright (C) 2005 Bill Marquette <bill.marquette@gmail.com>.
-    Copyright (C) 2003-2005 Manuel Kasper <mk@neon1.net>.
-    Copyright (C) 2004-2005 Scott Ullrich <sullrich@gmail.com>
-    All rights reserved.
-
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
-
-    1. Redistributions of source code must retain the above copyright notice,
-       this list of conditions and the following disclaimer.
-
-    2. Redistributions in binary form must reproduce the above copyright
-       notice, this list of conditions and the following disclaimer in the
-       documentation and/or other materials provided with the distribution.
-
-    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
-    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-    AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
-    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
-*/
+ * Copyright (C) 2014-2015 Deciso B.V.
+ * Copyright (C) 2005 Bill Marquette <bill.marquette@gmail.com>
+ * Copyright (C) 2003-2005 Manuel Kasper <mk@neon1.net>
+ * Copyright (C) 2004-2005 Scott Ullrich <sullrich@gmail.com>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
 
 require_once("guiconfig.inc");
 require_once("interfaces.inc");
@@ -89,6 +89,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     if (isset($pconfig['id']) && isset($a_vip[$pconfig['id']])) {
         $id = $pconfig['id'];
     }
+    $selected_interface = config_read_array('interfaces', $pconfig['interface']);
+    $selected_interface = !empty($selected_interface) ? $selected_interface['if'] : $pconfig['interface'] ;
 
     // perform form validations
     $reqdfields = array("mode");
@@ -110,29 +112,19 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             }
         }
         if (!empty($pconfig['gateway']) && !is_ipaddr($pconfig['gateway'])) {
-            $input_errors[] = gettext("A valid IP address must be specified.");
-        }
-
-        $natiflist = get_configured_interface_with_descr();
-        foreach ($natiflist as $natif => $natdescr) {
-            if ($pconfig['interface'] == $natif && (empty($config['interfaces'][$natif]['ipaddr']) && empty($config['interfaces'][$natif]['ipaddrv6']))) {
-                $input_errors[] = gettext("The interface chosen for the VIP has no IPv4 or IPv6 address configured so it cannot be used as a parent for the VIP.");
-            }
+            $input_errors[] = gettext("A valid gateway IP address must be specified.");
         }
 
         /* ipalias and carp should not use network or broadcast address */
         if ($pconfig['mode'] == "ipalias" || $pconfig['mode'] == "carp") {
-            if (is_ipaddrv4($pconfig['subnet']) && $pconfig['subnet_bits'] != "32") {
+            if (is_ipaddrv4($pconfig['subnet']) && $pconfig['subnet_bits'] != '32' && $pconfig['subnet_bits'] != '31') {
                 $network_addr = gen_subnet($pconfig['subnet'], $pconfig['subnet_bits']);
                 $broadcast_addr = gen_subnet_max($pconfig['subnet'], $pconfig['subnet_bits']);
-            } else if (is_ipaddrv6($pconfig['subnet']) && $_POST['subnet_bits'] != "128" ) {
-                $network_addr = gen_subnetv6($pconfig['subnet'], $pconfig['subnet_bits']);
-                $broadcast_addr = gen_subnetv6_max($pconfig['subnet'], $pconfig['subnet_bits']);
-            }
-            if (isset($network_addr) && $pconfig['subnet'] == $network_addr) {
-                $input_errors[] = gettext("You cannot use the network address for this VIP");
-            } else if (isset($broadcast_addr) && $pconfig['subnet'] == $broadcast_addr) {
-                $input_errors[] = gettext("You cannot use the broadcast address for this VIP");
+                if (isset($network_addr) && $pconfig['subnet'] == $network_addr) {
+                    $input_errors[] = gettext("You cannot use the network address for this VIP");
+                } else if (isset($broadcast_addr) && $pconfig['subnet'] == $broadcast_addr) {
+                    $input_errors[] = gettext("You cannot use the broadcast address for this VIP");
+                }
             }
         }
 
@@ -142,7 +134,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         if ($pconfig['mode'] == 'carp') {
             /* verify against reusage of vhids */
             foreach($config['virtualip']['vip'] as $vipId => $vip) {
-               if (isset($vip['vhid']) &&  $vip['vhid'] == $pconfig['vhid'] && $vip['interface'] == $pconfig['interface'] && $vipId <> $id) {
+               if (isset($vip['vhid']) &&  $vip['vhid'] == $pconfig['vhid'] && $vip['mode'] == 'carp' && $vip['interface'] == $pconfig['interface'] && $vipId != $id) {
                    $input_errors[] = sprintf(gettext("VHID %s is already in use on interface %s. Pick a unique number on this interface."),$pconfig['vhid'], convert_friendly_interface_to_friendly_descr($pconfig['interface']));
                }
             }
@@ -152,11 +144,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             if (empty($pconfig['vhid'])) {
                $input_errors[] = gettext('A VHID must be selected for this CARP VIP.');
             }
-            if ($pconfig['interface'] == "lo0") {
-                $input_errors[] = gettext("For this type of vip localhost is not allowed.");
+            if (substr($selected_interface,0, 2) === 'lo') {
+                $input_errors[] = gettext('For this type of VIP loopback is not allowed.');
             }
-        } else if ($pconfig['mode'] != 'ipalias' && $pconfig['interface'] == "lo0") {
-            $input_errors[] = gettext("For this type of vip localhost is not allowed.");
+        } else if ($pconfig['mode'] != 'ipalias' && substr($selected_interface,0, 2) === 'lo') {
+            $input_errors[] = gettext('For this type of VIP loopback is not allowed.');
         } elseif ($pconfig['mode'] == 'ipalias' && !empty($pconfig['vhid'])) {
             $carpvip_found = false;
             foreach($config['virtualip']['vip'] as $vipId => $vip) {
@@ -188,7 +180,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $vipent['noexpand'] = true;
         }
 
-        // virtual ip UI keeps track of it's changes in a separate file
+        // virtual ip UI keeps track of its changes in a separate file
         // (which is only use on apply in firewall_virtual_ip)
         // add or change this administration here.
         // Not the nicest thing to do, but we keep it for now.
@@ -306,7 +298,7 @@ $( document ).ready(function() {
 
 </script>
 
-</script>
+
   <section class="page-content-main">
     <div class="container-fluid">
       <div class="row">
@@ -343,11 +335,11 @@ $( document ).ready(function() {
                     <td>
                       <select name="interface" class="selectpicker" data-width="auto">
 <?php
-                      $interfaces = get_configured_interface_with_descr(false, true);
-                      $interfaces['lo0'] = "Localhost";
-                      foreach ($interfaces as $iface => $ifacename): ?>
-                        <option value="<?=$iface;?>" <?= $iface == $pconfig['interface'] ? "selected=\"selected\"" :""; ?>>
-                          <?=htmlspecialchars($ifacename);?>
+                      $interfaces = legacy_config_get_interfaces(array('virtual' => false));
+                      $interfaces['lo0'] = array('descr' => 'Loopback');
+                      foreach ($interfaces as $iface => $ifcfg): ?>
+                        <option value="<?=$iface;?>" <?= $iface == $pconfig['interface'] ? 'selected="selected"' : '' ?>>
+                          <?= htmlspecialchars($ifcfg['descr']) ?>
                         </option>
 <?php
                       endforeach; ?>
@@ -373,7 +365,7 @@ $( document ).ready(function() {
                   <tr>
                       <td><a id="help_for_address" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Address");?></td>
                       <td>
-                        <table style="border:0; cellspacing:0; cellpadding:0">
+                        <table style="border:0;">
                           <tr>
                             <td style="width:348px">
                               <input name="subnet" type="text" class="form-control" id="subnet" size="28" value="<?=$pconfig['subnet'];?>" />
@@ -480,8 +472,8 @@ $( document ).ready(function() {
                   <tr>
                     <td>&nbsp;</td>
                     <td>
-                      <input name="Submit" type="submit" class="btn btn-primary" value="<?=gettext("Save"); ?>" />
-                      <input type="button" class="btn btn-default" value="<?=gettext("Cancel");?>" onclick="window.location.href='/firewall_virtual_ip.php'" />
+                      <input name="Submit" type="submit" class="btn btn-primary" value="<?=html_safe(gettext('Save')); ?>" />
+                      <input type="button" class="btn btn-default" value="<?=html_safe(gettext('Cancel'));?>" onclick="window.location.href='/firewall_virtual_ip.php'" />
                       <?php if (isset($id) && $a_vip[$id]): ?>
                         <input name="id" type="hidden" value="<?=$id;?>" />
                       <?php endif; ?>
